import { ApiHideProperty, ApiProperty } from '@nestjs/swagger';
import { Column, Entity, JoinTable, ManyToMany, Relation } from 'typeorm';
import { CommonEntity } from '~/common/entity/common.entity';
import { Storage } from '../tools/storage/storage.entity';

@Entity({ name: 'contract' })
export class ContractEntity extends CommonEntity {
  @Column({
    name: 'contract_number',
    type: 'varchar',
    length: 255,
    unique: true,
    comment: '合同编号'
  })
  @ApiProperty({ description: '合同编号' })
  contractNumber: string;

  @Column({ name: 'title', type: 'varchar', length: 255, comment: '合同标题' })
  @ApiProperty({ description: '合同标题' })
  title: string;

  @Column({ type: 'int', comment: '合同类型(字典)' })
  @ApiProperty({ description: '合同类型(字典)' })
  type: number;

  @Column({ name: 'party_a', length: 255, type: 'varchar', comment: '甲方' })
  @ApiProperty({ description: '甲方' })
  partyA: string;

  @Column({ name: 'party_b', length: 255, type: 'varchar', comment: '乙方' })
  @ApiProperty({ description: '乙方' })
  partyB: string;

  @Column({ name: 'signing_date', type: 'date', nullable: true })
  @ApiProperty({ description: '签订日期' })
  signingDate: Date;

  @Column({ name: 'delivery_deadline', type: 'date', nullable: true })
  @ApiProperty({ description: '交付期限' })
  deliveryDeadline: Date;

  @Column({ name: 'status', type: 'tinyint', default: 0, comment: '审核状态(字典)' })
  @ApiProperty({ description: '审核状态：0待审核，1同意，2.不同意(字典)' })
  status: number;

  @ManyToMany(() => Storage, storage => storage.contracts)
  @JoinTable({
    name: 'contract_storage',
    joinColumn: { name: 'contract_id', referencedColumnName: 'id' },
    inverseJoinColumn: { name: 'file_id', referencedColumnName: 'id' }
  })
  files: Relation<Storage[]>;
}
